package se.salomonsson.g2d;

import java.util.ArrayList;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Rect;
import android.util.Log;

public class GameThread implements Runnable {

	private Thread mThread;
	private GameView mGameView;
	private boolean mRunning;
	
	private ArrayList<IUpdateable> mUpdateQueue;
	private ArrayList<IRenderable> mRenderQueue;
	
	public GameThread( GameView gv )
	{
		mGameView = gv;
		mThread = new Thread(this);
		mThread.setName("GameThread");
		
		mUpdateQueue = new ArrayList<IUpdateable>();
		mRenderQueue = new ArrayList<IRenderable>();
		
		mBackgroundColor = new Paint();
		mBackgroundColor.setARGB(255, 0, 0, 0);
	}
	
	public void start()
	{
		if (!mRunning)
		{
			mThread.start();
		}
	}
	
	public void stop()
	{
		mRunning = false;
	}
	
	
	
	private Bitmap mBitmap;
	private Canvas mCanvas;
	private Dim mLastDim;
	private Rect mCamera;
	private Paint mBackgroundColor;
	
	
	
	public void addUpdateable(IUpdateable obj)
	{
		if (mUpdateQueue.indexOf(obj) < 0)
			mUpdateQueue.add(obj);
	}
	
	public void removeUpdateable(IUpdateable obj)
	{
		mUpdateQueue.remove(obj);
	}
	
	public void addRenderable(IRenderable obj)
	{
		if (mRenderQueue.indexOf(obj) < 0)
			mRenderQueue.add(obj);
	}
	
	public void removeRenderable(IRenderable obj)
	{
		mRenderQueue.remove(obj);
	}
	
	@Override
	public void run() {
		
		mRunning = true;
		
		while (mRunning) {
			
			synchronized (mGameView) {
				if (!mGameView.getDim().equals(mLastDim))
				{
					mLastDim = mGameView.getDim();
					mCamera = new Rect(0,0,mLastDim.getW(),mLastDim.getH());
					
					if (!mLastDim.isEmpty())
					{
						mBitmap = Bitmap.createBitmap(mLastDim.getW(), mLastDim.getH(), Bitmap.Config.ARGB_8888);
						mCanvas = new Canvas(mBitmap);
						
						Log.i("tommy", "run, dim=" + mLastDim.getW() + ", " + mLastDim.getH());
					}
				}
				
				
				if (mCanvas != null)
				{
					update();
					render(mCanvas);
					
					mGameView.draw(mBitmap);
				}
			}
		}
	}
	
	protected void update()
	{
		for(IUpdateable updateable: mUpdateQueue)
			updateable.update(mCamera);
	}
	
	protected void render(Canvas c)
	{
		c.drawPaint(mBackgroundColor);
		
		for(IRenderable render:mRenderQueue)
			render.render(c, mCamera);
	}

}
