package se.salomonsson.g2d;

import se.salomonsson.observer.Observable;
import se.salomonsson.utils.Util;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;

import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

public class GameView extends SurfaceView implements SurfaceHolder.Callback
{

	private Observable mSizeObserver;
	private Observable mSurfaceHolderObserver;
	
	private SurfaceHolder mHolder;
	private Dim mDim;
	//private int mFormat;
	
	public Observable getSizeObserver() { return mSizeObserver; }
	public Observable getSurfaceHolderObserver() { return mSurfaceHolderObserver; }
	
	public SurfaceHolder getSurfaceHolder() { return mHolder; }
	public Dim getDim() { return mDim; }
	
	
	public GameView(Context context)
	{
		super(context);
		
		Util.preventDimOnView(this);
		
		mSizeObserver = new Observable();
		mSurfaceHolderObserver = new Observable();
		
		mDim = new Dim(0,0,0,0);
		this.getHolder().addCallback(this);
	}
	
	@Override
	public void surfaceChanged( SurfaceHolder holder, int format, int width, int height )
	{
		mDim = new Dim(0,0,width,height);
		//mFormat = format;
		mSizeObserver.dispatch();
		
		Log.i("GameView", "GameView.surfaceChanged");
	}

	@Override
	public void surfaceCreated(SurfaceHolder holder) 
	{
		mHolder = holder;
		mSurfaceHolderObserver.dispatch();
		
		Log.i("GameView", "GameView.surfaceCreated");
	}

	@Override
	public void surfaceDestroyed(SurfaceHolder holder) 
	{
		mHolder = null;
		mDim = new Dim(0,0,0,0);
		mSizeObserver.dispatch();
		
		Log.i("tommy", "GameView.surfaceDestroyed");
	}
	
	public void draw(Bitmap b)
	{
		if (mHolder != null)
		{
			Canvas c = mHolder.lockCanvas();
			if (c != null)
			{
				c.drawBitmap(b, 0, 0, null);
				mHolder.unlockCanvasAndPost(c);
			}
			//Log.i("tommy","GameView.draw() " + c);
		}
	}
}
